/* -*- Mode: C; tab-width: 4; indent-tabs-mode: s; c-basic-offset: 4 -*- */
/***************************************************************************
 *
 * Copyright (C) 2009 Alex Dorokhov, <alex.dorokhov@gmail.com>
 * Copyright (C) 2008 Dan Williams, <dcbw@redhat.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 **************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <sys/types.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include <glib.h>
#include <glib/gi18n-lib.h>
#include <glade/glade.h>

#include <nm-connection.h>
#include <nm-setting-vpn.h>

#include "advanced-dialog.h"
#include "nm-openswan.h"
#include "../src/nm-openswan-service.h"


static const char *advanced_keys[] = {
	OPENSWAN_PLUGIN_KEY_NAT,
	OPENSWAN_PLUGIN_KEY_DPD,
	OPENSWAN_PLUGIN_KEY_IFACE,
	NULL
};

static GList* get_interfaces_list()
{
    gchar *argv[] = {"/usr/bin/python", NM_OPENSWAN_SERVICE_DIR"/properties_utils.py", "--list-interfaces" ,NULL};
    gchar *envs[] = { NULL };
    gchar *wd = ".";
    GError *error = NULL;
    gint sout;
    GPid child_pid;
    if(g_spawn_async_with_pipes(wd, argv, envs, 0, NULL, NULL,
                                &child_pid, NULL, &sout, NULL,
                                &error)){
        GList* result = NULL;
        char buf;
        GString* strres = g_string_new("");
        while(read(sout, &buf, 1) > 0){
            if(buf != '\n'){
                strres = g_string_append_c(strres, buf);
            } else {
                result = g_list_append(result, strres);
                strres = g_string_new("");
            }
        }
        close(sout);
        g_spawn_close_pid(child_pid);
        return result;
    }
    return NULL;
}

static void
copy_values (const char *key, const char *value, gpointer user_data)
{
	GHashTable *hash = (GHashTable *) user_data;
	const char **i;

	for (i = &advanced_keys[0]; *i; i++) {
		if (strcmp (key, *i))
			continue;
		g_hash_table_insert (hash, g_strdup (key), g_strdup (value));
	}
}

GHashTable *
advanced_dialog_new_hash_from_connection (NMConnection *connection,
                                          GError **error)
{
	GHashTable *hash;
	NMSettingVPN *s_vpn;

	hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);

	s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
	nm_setting_vpn_foreach_data_item (s_vpn, copy_values, hash);
	return hash;
}

GtkWidget *
advanced_dialog_new (GHashTable *hash)
{
	GladeXML *xml;
	GtkWidget *dialog = NULL;
	char *glade_file = NULL;
	GtkWidget *widget;
	const char *value;

	g_return_val_if_fail (hash != NULL, NULL);

	glade_file = g_strdup_printf ("%s/%s", GLADEDIR, "nm-openswan-dialog.glade");
	xml = glade_xml_new (glade_file, "advanced-network-dialog", GETTEXT_PACKAGE);
	if (xml == NULL)
		goto out;

	dialog = glade_xml_get_widget (xml, "advanced-network-dialog");
	if (!dialog) {
		g_object_unref (G_OBJECT (xml));
		goto out;
	}
	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);

	g_object_set_data_full (G_OBJECT (dialog), "glade-xml",
	                        xml, (GDestroyNotify) g_object_unref);
	
	// Throw interface init
	widget = glade_xml_get_widget (xml, "iface-cb");
	value = g_hash_table_lookup (hash, OPENSWAN_PLUGIN_KEY_IFACE);
	GList* iface_list = get_interfaces_list();
	GList* cur;
	if((!value) || (g_strcmp0 (value, OPENSWAN_PLUGIN_IFACE_DEFAULT) == 0)){
		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 0);
		for(cur = iface_list; cur != NULL; cur = cur->next){
			gchar *data = g_string_free(((GString*)cur->data), FALSE);
			gtk_combo_box_append_text (GTK_COMBO_BOX (widget), data);
		}
	} else {
		int active = -1;
		int i = 1; // 0 - is default
		for(cur = iface_list; cur != NULL; cur = cur->next){
			gchar *data = g_string_free(((GString*)cur->data), FALSE);
			gtk_combo_box_append_text (GTK_COMBO_BOX (widget), data);
			if(g_strcmp0 (value, data) == 0){
		    	active = i;
			}
			i++;
		}
		if(active < 0){
			active = i;
			gtk_combo_box_append_text (GTK_COMBO_BOX (widget), value);
		}
		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), active);
	}
	
	// NAT-Traversal init
	widget = glade_xml_get_widget (xml, "nat-chb");
	value = g_hash_table_lookup (hash, OPENSWAN_PLUGIN_KEY_NAT);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), (value && !strcmp (value, "yes")));
	
	// Dead Peer Detection init
	widget = glade_xml_get_widget (xml, "dpd-chb");
	value = g_hash_table_lookup (hash, OPENSWAN_PLUGIN_KEY_DPD);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), (value && !strcmp (value, "yes")));

out:
	g_free (glade_file);
	return dialog;
}

GHashTable *
advanced_dialog_new_hash_from_dialog (GtkWidget *dialog, GError **error)
{
	GHashTable *hash;
	GtkWidget *widget;
	GladeXML *xml;

	g_return_val_if_fail (dialog != NULL, NULL);
	if (error)
		g_return_val_if_fail (*error == NULL, NULL);

	xml = g_object_get_data (G_OBJECT (dialog), "glade-xml");
	g_return_val_if_fail (xml != NULL, NULL);

	hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);

	widget = glade_xml_get_widget (xml, "iface-cb");
	if(gtk_combo_box_get_active(GTK_COMBO_BOX (widget)) == 0) {
		g_hash_table_insert (hash, g_strdup (OPENSWAN_PLUGIN_KEY_IFACE), OPENSWAN_PLUGIN_IFACE_DEFAULT);
	} else {
		g_hash_table_insert (hash, g_strdup (OPENSWAN_PLUGIN_KEY_IFACE), 
			gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget)));
	}

	widget = glade_xml_get_widget (xml, "nat-chb");
	g_hash_table_insert (hash, g_strdup (OPENSWAN_PLUGIN_KEY_NAT),
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) ? g_strdup ("yes") : g_strdup ("no"));

	widget = glade_xml_get_widget (xml, "dpd-chb");
	g_hash_table_insert (hash, g_strdup (OPENSWAN_PLUGIN_KEY_DPD),
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) ? g_strdup ("yes") : g_strdup ("no"));
	

	return hash;
}

